import requests
import sys
from datetime import datetime

# config
profile_set_name = 'Standard'
server = "192.168.178.72"
address = 'http://{}/masking/api'.format(server)
username = 'admin'
password = 'Admin-12'
# for engine older than 6.0.15 set to False so typConstraints are not executed
enable_type_expressions = True
log_file = 'scriptlog.txt'
profile_set_file = 'domains.csv'
console_logging = 0
file_logging = 0
# if 1 the default algo of domain is updated to setting in config > otherwise default algo unchanged
update_domain = 1


def put(url, post_json=None, authorization=None):
    headers = {
        'Content-Type': 'application/json',
        'Accept': 'application/json',
        'Authorization': authorization
    }
    r = requests.put(url, json=post_json, headers=headers)
    return r.json()


def post(url, post_json, authorization=None):
    headers = {
        'Content-Type': 'application/json',
        'Accept': 'application/json'
    }
    if authorization:
        headers["Authorization"] = authorization
    r = requests.post(url, json=post_json, headers=headers)
    return r.json()


def get(url, authorization, post_json=None):
    headers = {
        'Content-Type': 'application/json',
        'Accept': 'application/json',
        'Authorization': authorization
    }
    url = url + '?page_size=1000'
    r = requests.get(url, json=post_json, headers=headers )
    return r.json()


def delete(url, authorization):
    headers = {
        'Content-Type': 'application/json',
        'Accept': 'application/json',
        'Authorization': authorization
    }
    r = requests.delete(url, headers=headers)
    return r.status_code


def clean():
    log_me("Started cleaning: {}".format(datetime.now().strftime("%d.%m.%Y, %H:%M:%S")), 1)
    key = authenticate()
    clean_csv(key)
    log_me("Finished cleaning: {}".format(datetime.now().strftime("%d.%m.%Y, %H:%M:%S")), 1)


def process():
    log_me("Started processing: {}".format(datetime.now().strftime("%d.%m.%Y, %H:%M:%S")), 1)
    key = authenticate()
    process_csv(key)
    log_me("Finished processing: {}".format(datetime.now().strftime("%d.%m.%Y, %H:%M:%S")), 1)


def main():
    print(f"Arguments count: {len(sys.argv)}")
    for i, arg in enumerate(sys.argv):
        print(f"Argument {i:>6}: {arg}")
    if sys.argv[1] == "CLEAN":
        clean()
    elif sys.argv[1] == "PROCESS":
        process()
    elif sys.argv[1] == "REPORT":
        report()
    else:
        print("Invalid or missing argument > require PROCESS / CLEAN / REPORT ")


def authenticate():
    mylogin = {'username': username, 'password': password}
    authkey_response = post(address + "/login", mylogin)
    log_me("response: {}".format(authkey_response))
    log_me("Authentication Key: " + authkey_response["Authorization"])
    key = authkey_response["Authorization"]
    return key


def log_me(str, force_log=0):
    if console_logging or force_log > 0:
        print(str)
    if force_log > 0 or file_logging > 0:
        with open("{}".format(datetime.now().strftime("%d_%m_%Y") + "_" + log_file), 'a') as a_writer:
            a_writer.write('\n{}'.format(str))


def get_profile_type_expressions(key):
    profile_expressions = get(address + "/profile-type-expressions", key)
    if 'responseList' in profile_expressions:
        return profile_expressions["responseList"]
    else:
        return []


def get_profile_expressions(key):
    profile_expressions = get(address + "/profile-expressions", key)
    if 'responseList' in profile_expressions:
        return profile_expressions["responseList"]
    else:
        return []


def report():
    key = authenticate()
    set = get_profile_set(key)
    log_me("PROFILE SET: {}".format(set['profileSetName']))
    log_me("\tCOLUMN EXPRESSIONS: {}".format(set['profileExpressionIds']))
    log_me("\tTYPE   EXPRESSIONS: {}".format(set['profileTypeExpressionIds']))


def process_csv(key):
    # get default list for expressions and typeExpressions
    global global_profile_expressions_list
    global_profile_expressions_list = get_profile_expressions(key)
    log_me("global_profile_expressions_list : {}".format(global_profile_expressions_list), 1)

    global global_domain_list
    global_domains = get(address + "/domains", key)
    global_domain_list = global_domains["responseList"]

    if enable_type_expressions:
        global global_profile_type_expressions_list
        global_profile_type_expressions_list = get_profile_type_expressions(key)

    global addTypeExpressionList
    addTypeExpressionList = []
    global addExpressionList
    addExpressionList = []

    with open(profile_set_file) as file:
        lines = file.readlines()
    log_me("Rows to process: {}".format(len(lines)), 1)
    for line in lines:
        line = line.strip()
        if line[:1] == "#":
            log_me("comment from config: {}".format(line), 1)
            continue
        conf = line.split(",")
        if len(conf) != 4:
            log_me("Configuration row {} is invalid (must be 4 arguments)".format(conf), 1)
            continue

        if conf[1] == "MA":
            process_domain(key, conf)
        if conf[1] == "CE":
            process_profile_expression(key, conf, 0)
        if conf[1] == "DE" and enable_type_expressions:
            process_profile_expression(key, conf, 1)
        if conf[1] == "TE" and enable_type_expressions:
            process_profile_type_expression(key, conf)

    process_profile_set(key)


def process_domain(key, conf):
    # check if domain exists
    get_domains = get(address + "/domains/{}".format(conf[0]), key)
    log_me("DOMAIN : {}".format(conf), 1)
    if str(get_domains).find("find Domain") == -1:
        log_me("\tDomain {} exists".format(conf[0]), 1)
        if str(get_domains["defaultAlgorithmCode"]) != conf[3]:
            update_domain_body = {'domainName': conf[0], 'defaultAlgorithmCode': conf[3]}
            update_domain_response = put(address + "/domains/{}".format(conf[0]), update_domain_body, key)
            log_me("\tDomain update response: " + str(update_domain_response))
    else:
        log_me("\tDomain {} does not exists".format(conf[0]), 1)
        post_domains_body = {'domainName': conf[0], 'defaultAlgorithmCode': conf[3]}
        post_domains_response = post(address + "/domains", post_domains_body, key)
        log_me("\tPOST domains response: " + str(post_domains_response))


def process_profile_expression(key,conf,data):
    log_me("PROFILE COLUMN EXPRESSION: {}".format(conf))
    pe_exists = 0
    for profile_expression in global_profile_expressions_list:
        if profile_expression['expressionName'] == conf[2]:
            log_me("\tprofile_expression {} exists".format(conf[2]))
            pe_exists = 1
            break
    pe_body = {'domainName': conf[0],
               'expressionName': conf[2],
               'regularExpression': conf[3],
               'dataLevelProfiling': data }
    if pe_exists == 1:
        pe_response = put(address + "/profile-expressions/{}".format(profile_expression['profileExpressionId']),pe_body,key)
        log_me("\tUpdate profile-expression response: {}".format(pe_response))
        addExpressionList.append(pe_response['profileExpressionId'])

    else:
        pe_response = post(address + "/profile-expressions", pe_body, key)
        log_me("\tCreate profile-expression response: {}".format(pe_response))
        addExpressionList.append(pe_response['profileExpressionId'])


def process_profile_type_expression(key,conf):
    pte_exists = 0
    dataType = conf[3].split(":")[0]
    typeExpressionName = conf[0] + "_" + dataType
    minDataLength = 0

    if len(conf[3].split(":")) > 1:
        minDataLength = int(conf[3].split(":")[1])

    log_me("PROFILE TYPE EXPRESSION: {}".format(conf))
    for typeExpression in global_profile_type_expressions_list:
        if typeExpression['typeExpressionName'] == typeExpressionName:
            log_me("\tprofile_type_xpression {} exists".format(typeExpression))
            pte_id = typeExpression['profileExpressionId']
            pte_exists = 1
            break

    if minDataLength > 0:
        pte_body = {'domainName': conf[0],
                    'typeExpressionName': typeExpressionName,
                    'dataType': dataType,
                    'dataLevelProfiling': 'false',
                    'minDataLength': minDataLength
                    }
    else:
        pte_body = {'domainName': conf[0],
                    'typeExpressionName': typeExpressionName,
                    'dataType': dataType,
                    'dataLevelProfiling': 'false',
                    }

    if pte_exists:
        pte_response = put(address + "/profile-type-expressions/{}".format(pte_id), pte_body, key)
        log_me("\tPUT profile-type-expressions response for conf[5]: {}".format(pte_response))
        # exists already > we only update but don't add
        # #addTypeExpressionList.append(pte_response['profileExpressionId'])
    else:
        pte_response = post(address + "/profile-type-expressions", pte_body, key)
        log_me("\tPOST profile-type-expressions response for conf[5]: {}".format(pte_response))
        addTypeExpressionList.append(pte_response['profileExpressionId'])


def get_profile_set(key):
    profile_sets = get(address + "/profile-sets", key)
    if "responseList" in profile_sets:
        profile_set_list = profile_sets["responseList"]
    else:
        profile_set_list = []
    for set in profile_set_list:
        if set["profileSetName"] == profile_set_name:
            return set


def process_profile_set(key):
    profile_sets = get(address + "/profile-sets", key)
    if "responseList" in profile_sets:
        profile_set_list = profile_sets["responseList"]
    else:
        profile_set_list = []
    exists = 0
    log_me("PROFILE_SET : {}".format(profile_set_name), 1)
    for set in profile_set_list:
        if set["profileSetName"] == profile_set_name:
            log_me("\tprofileSet {} exists".format(profile_set_name))
            exists = 1
            break

    if exists == 0:
        profile_set_body = {'profileSetName': profile_set_name,
                            'profileExpressionIds': addExpressionList,
                            'profileTypeExpressionIds': addTypeExpressionList
                            }
        log_me("\tUPDATE profileSetBody: {}".format(profile_set_body))
        profile_set_response = post(address + "/profile-sets", profile_set_body, key)
        log_me("\tPOST profile-set: " + profile_set_name + " created with response: {}".format(profile_set_response))
    else:
        #profileset does not exist
        for profile_set in profile_set_list:
            if profile_set_name == profile_set["profileSetName"]:
                if "profileExpressionIds" in profile_set:
                    profile_expression_ids = profile_set["profileExpressionIds"]
                else:
                    profile_expression_ids = []
                for expression in addExpressionList:
                    if expression in profile_expression_ids:
                        continue
                    else:
                        profile_expression_ids.append(expression)

                if "profileTypeExpressionIds" in profile_set:
                    profile_type_expression_ids = profile_set["profileTypeExpressionIds"]
                else:
                    profile_type_expression_ids = []
                for expression in addTypeExpressionList:
                    if expression in profile_type_expression_ids:
                        continue
                    else:
                        profile_type_expression_ids.append(expression)
                log_me(
                    "\tUpdating profile set " + profile_set_name +
                    "\n\t\t with Expression list: {}".format(profile_expression_ids) +
                    "\n\t\t with typeExpression list: {} ".format(profile_type_expression_ids),
                    1)

                profile_set_body = {'profileSetName': profile_set_name,
                                    'profileExpressionIds': profile_expression_ids,
                                    'profileTypeExpressionIds': profile_type_expression_ids
                                    }
                log_me("\tUPDATE profile_set_body: {}".format(profile_set_body), 1)
                update_profile_set_response = put(address + "/profile-sets/{}".format(profile_set['profileSetId']),
                                                  profile_set_body, key)
                log_me("\tUPDATE profile_set " + profile_set_name + " with response: {}".format(update_profile_set_response))


def clean_csv(key):
    # delete_profile_job(key, job_name)
    # delete_profile_set(key, profile_set_name)
    delete_all_profile_expressions(key)
    delete_all_profile_type_expressions(key)
    profile_type_expressions = get(address + "/profile-type-expressions", key)
    profile_expressions = get(address + "/profile-expressions", key)
    with open(profile_set_file) as file:
        lines = file.readlines()
    # header = content[:1]
    # log_me(header)
    # lines = content[1:]
    log_me("Rows to process: {}".format(len(lines)), 1)
    for line in lines:
        line = line.strip()
        if line[:1] == "#":
            log_me("comment from config: {}".format(line), 1)
            continue
        conf = line.split(",")
        if len(conf) != 4:
            log_me("Configuration row {} is incomplete (exactly 4 arguments required)".format(conf), 1)
            continue

        if conf[1] == "MA":
            delete_domain(key, conf)

        if conf[1] == "CE":
            delete_profile_expression(key, conf[2], profile_expressions)
        #if conf[1] == "TE":
        #    delete_profile_type_expression(key, conf[2], profile_type_expressions)



def delete_profile_set(key, profile_set_name):
    profile_sets = get(address + "/profile-sets", key)
    profile_list = profile_sets["responseList"]
    for profile in profile_list:
        if profile["profileSetName"] == profile_set_name:
            id = profile["profileSetId"]
            delete_profile_set_response = delete(address + "/profile-sets/{}".format(id), key)
            log_me("\tDeleted profile-set: {}".format(id) + " {}" + profile[
                "profileSetName"] + " with response: {}".format(delete_profile_set_response), 1)


def delete_domain(key, domain):
    delete_response = delete(address + "/domains/{}".format(domain), key)
    log_me("\tDeleted domain response: {}".format(delete_response), 1)


def delete_all_profile_type_expressions(key):
    log_me("Deleting all profile-type-expressions", 1)
    profile_type_expressions = get(address + "/profile-type-expressions", key)
    for id in profile_type_expressions["responseList"]:
        delete_type_response = delete(address + "/profile-type-expressions/{}".format(id["profileExpressionId"]), key)
        log_me(
            "\tDeleted profile-type-expression {}".format(id["profileExpressionId"]) + " typeExpressionName: {}".format(
                id["typeExpressionName"]) + " with response: {}".format(delete_type_response), 1)


def delete_profile_type_expression(key, typeExpressionName, profile_type_expressions):
    profile_type_expressions = get(address + "/profile-type-expressions", key)
    for id in profile_type_expressions["responseList"]:
        if str(id["typeExpressionName"]) == typeExpressionName:
            delete_type_response = delete(address + "/profile-type-expressions/{}".format(id["profileExpressionId"]), key)
            log_me("\tDeleted profile-type-expression {}".format(id["profileExpressionId"])
                   + " typeExpressionName: {}".format(id["typeExpressionName"]) + " with response: {}".format(delete_type_response), 1)


def delete_profile_job(key, job_name):
    profile_jobs = get(address + "/profile-jobs", key)
    job_list = profile_jobs["responseList"]
    for job in job_list:
        if job["jobName"] == job_name:
            id = job["profileJobId"]
            delete_job_response = delete(address + "/profile-jobs/{}".format(id), key)
            log_me("Deleted profile-job ID: {}".format(id) + " with response: {}".format(delete_job_response))


def delete_profile_expression(key, expressionName, profile_expressions ):
    for id in profile_expressions["responseList"]:
        if id['dataLevelProfiling']:
            return
        if id['createdBy'] == "System":
            return
        if str(id["expressionName"]) == expressionName:
            log_me("Deleting profile-expression: {}".format(expressionName), 1)
            delete_response = delete(address + "/profile-expressions/{}".format(id["profileExpressionId"]))
            log_me("\tDeleted profileExpressionId: {}".format(id["profileExpressionId"]) +
                   " expressionName: {}".format(id["expressionName"]) + " with response: {}".format(delete_response), 1)


def delete_all_profile_expressions(key):
    log_me("Deleting all profile-expressions", 1)
    profile_expressions = get(address + "/profile-expressions", key)
    for id in profile_expressions["responseList"]:
        if id['dataLevelProfiling'] == True:
            continue
        if id['createdBy'] == "System":
            continue
        delete_response = delete(address + "/profile-expressions/{}".format(id["profileExpressionId"]), key)
        log_me("\tDeleted profileExpressionId: {}".format(id["profileExpressionId"]) +
               " expressionName: {}".format(id["expressionName"]) + " with response: {}".format(delete_response), 1)


if __name__ == "__main__":
    main()
